#ifndef CGL_H
#define CGL_H



typedef enum { GL_MODELVIEW,
               GL_PROJECTION } matrix_mode;


typedef enum { GL_MODELVIEW_MATRIX,
               GL_PROJECTION_MATRIX } object_type;


typedef enum { GL_POINTS,
               GL_LINES,
               GL_LINE_STRIP,
               GL_LINE_LOOP,
               GL_TRIANGLES,
               GL_TRIANGLE_STRIP,
               GL_TRIANGLE_FAN,
               GL_QUADS,
               GL_QUAD_STRIP } render_style;



typedef float vertex3[3];
typedef float vertex4[4];
typedef float vector3[3];


typedef unsigned char COLOR;


typedef enum { BOTH_INSIDE,
               LEAVING,
               BOTH_OUTSIDE,
               ENTERING } line_status;


typedef enum { GL_CW,
               GL_CCW } face_winding;


typedef enum { GL_FRONT,
               GL_BACK,
               GL_FRONT_AND_BACK } face_mode;


typedef enum { SKIP = 0,
               GL_POINT,
               GL_LINE,
               GL_FILL } polygon_render_mode;


typedef enum { GL_CULL_FACE,
					GL_POLYGON_OFFSET_POINT,
					GL_POLYGON_OFFSET_LINE,
					GL_POLYGON_OFFSET_FILL
				 } server_side_capabilities;


enum{ white = 255,
      WHITE = 255,
      light = 170,
      LIGHT = 170,
      dark = 85,
      DARK = 85,
      black = 0,
      BLACK = 0 };


void glFrontFace( face_winding new_front_face_winding );



typedef enum { GL_FLAT,
					GL_SMOOTH
				} shade_model_T;


#define BACK_BUFFER_WIDTH (160 + 2)
#define BACK_BUFFER_HEIGHT (100 + 2)


#define GL_COLOR_BUFFER_BIT 0b1
#define GL_DEPTH_BUFFER_BIT 0b10







//////////// fl0at ////


#define POSITIVE 0
#define NEGATIVE 1

#define POS 0
#define NEG 1


typedef struct
{
	BOOL sign;
	char exponent;
	unsigned short mantissa;
} fl0at;



fl0at mv_proj_comb0_matrix[16];

fl0at m0delview_matrix[16];
fl0at pr0jection_matrix[16];


fl0at float_to_fl0at( float f );
float fl0at_to_float( fl0at f );



fl0at fladd(fl0at a, fl0at b);
fl0at flsub(fl0at a, fl0at b);
fl0at flmult(fl0at a, fl0at b);
fl0at fldiv(fl0at a, fl0at b);



BOOL fl_less_than(fl0at a, fl0at b);
BOOL fl_greater_than(fl0at a, fl0at b);

BOOL fl_less_than_or_equals(fl0at a, fl0at b);
BOOL fl_greater_than_or_equals(fl0at a, fl0at b);

BOOL fl_equals(fl0at a, fl0at b);

fl0at fl_abs(fl0at a);

fl0at flnegate( fl0at a );

fl0at make_fl0at( BOOL sign, char exponent, unsigned short mantissa );


short fl0at_to_short( fl0at f );
fl0at make_fl0at_s( short mantissa );


long fl0at_to_long( fl0at f );
fl0at make_fl0at_l( long mantissa );


typedef fl0at v3rtex3[3];
typedef fl0at v3rtex4[4];
typedef fl0at v3ctor3[3];



fl0at flnegate( fl0at a );


fl0at fl_ZERO;
fl0at	fl_ONE;


/////////// fl0at ///////







////////// shared ///////


short GL_INIT();

void GL_DEINIT();


void glPolygonMode( face_mode selected_face_mode,  polygon_render_mode new_render_mode );

void glPolygonOffset(long difference);

void glShadeModel(shade_model_T shade_model_in);

void glFrontFace( face_winding new_front_face_winding );	// can be GL_CW or GL_CCW

void glCullFace( face_mode new_cull_mode );

void glBegin(render_style mode);

void glEnd();

void glEnable( server_side_capabilities capability );

void glDisable( server_side_capabilities capability );

void glVertex3f(float x, float y, float z);
void glVertex3s(short x, short y, short z);
void glVertex3l(short x, short y, short z);
void glVertex3f0(fl0at x, fl0at y, fl0at z);


/////////// shared /////







/////////// math ///


void matrix_vect0r_multiply(fl0at* out_vector_4, fl0at* in_matrix_4x4, fl0at* in_vector_4);

void matrix_multiply(float* result, float* A, float* B);
void m4trix_multiply(fl0at* result, fl0at* A, fl0at* B);

float magnitude_3(float* vector_3); // returns the magnitude of the 3 element vector

void normalize_3(float* result_3, float* A_3);	// result = A / |A|

void cross_product_3(float* result_3, float* A_3, float* B_3);
void cross_pr0duct_3(fl0at* result_3, fl0at* A_3, fl0at* B_3);

fl0at dot_pr0duct_3(fl0at* A_3, fl0at* B_3);

void vector_3_divide(float* result_3, float* A_3, float denominator);  // result = A_3 / denominator
void vect0r_3_divide(fl0at* result_3, fl0at* A_3, fl0at denominator);

void vector_3_subtract(float* result_3, float* A_3, float* B_3);  // result = A - B
void vect0r_3_subtract(fl0at* result_3, fl0at* A_3, fl0at* B_3);





/////////// math ////



///////// matrix operations ////




void glGetFloatv(object_type requested_object, float* result);

void glMatrixMode(matrix_mode new_matrix_mode);  // sets the current matrix mode and the pointer to the current matrix

void glMultMatrixf(float* in_matrix);

void glLoadMatrixf(float* in_matrix);

void glScalef(float x, float y, float z);

void glTranslatef(float x, float y, float z);

void glRotatef(float angle, float x, float y, float z);

void gluLookAt(float eyeX, float eyeY, float eyeZ,
					float centerX, float centerY, float centerZ,
					float upX, float upY, float upZ);

void glLoadIdentity();

void gluPerspective(float fovy, float aspect, float zNear, float zFar);


void glViewport(short int x, short int  y, short int  width, short int  height);

void glFrustum(float left, float right, float bottom, float top, float zNear, float zFar);

void glOrtho(float left, float right, float bottom, float top, float zNear, float zFar);

void glPushMatrix();

void glPopMatrix();


///////// matrix operations ////////



//////// buffers //////////



void glClearDepth(float depth);

void glClear(unsigned short mask);

void glClearColor(COLOR color);

void glColor(COLOR color);

void SwapBuffers();


////////// buffers /////////////







#endif












